import os.path
import requests
from urllib.parse import urljoin, urlparse

def html(url):
    """
    Retourne le contenu html (chaîne de caractère) d'une page web identifiée par son URL ou ""
    si aucun contenu texte n'est trouvé.
    """
    try:
        r = requests.get(url)
        if r.status_code != 200 or "text/html" not in r.headers["Content-type"]:
            return ""
        return r.text
    except: # Timeout ou autre joyeuseté, lien mort ?
        return ""

def normalise_urls(L, url):
    """
    Retourne la liste des URL absolue et filtre les liens non http.
    """
    L2 = []
    for u in L:
        if u != "":
            res = urljoin(url, u)
            if urlparse(res).scheme in ("http", "https"):
                L2.append(res)
    return L2

def prochaine_occ(motif, texte, i):
    """
    Retourne l'indice (supérieur ou égal à i) du premier caractère de texte
    tel que le début de text[i:] soit exactement motif, c'est à dire l'indice du début de
    la prochaine occurence de motif dans la chaîne texte.

    Retourne -1 si aucune occurence n'est trouvée après l'indice i.
    """
    j = i
    n = len(motif)
    m = len(texte)
    while j <= m - n:
        if texte[j: j + n] == motif:
            return j
        j = j + 1
    return -1


def prochaine_url(texte, i):
    """
    Retourne la première url trouvée dans texte après l'indice i ainsi que l'indice du
    premier caractère après cette url.

    Si aucune url n'est trouvée, retourne "", -1
    """
    j = prochaine_occ("href=", texte, i)
    if j == -1:
        return "", -1
    j = j + 5  # on se place après le =
    # teste pour savoir le type de guillemet
    if texte[j] == '"':
        fin = prochaine_occ('"', texte, j + 1)
    elif texte[j] == "'":
        fin = prochaine_occ("'", texte, j + 1)
    else:
        fin = j + 1
    if fin == -1:
        return "", -1
    return texte[j + 1: fin], fin + 1


EXTENSION_WL = [".htm", ".html", ".php", ".php3"]

def est_action(url):
    """Indique si un lien renvoie vers une action comme envoyer un e-mail, téléphoner..."""
    b = url.startswith("tel:")
    b = b or url.startswith("mailto:")
    b = b or url.startswith("fax:")
    b = b or url.startswith("callto:") # appel skype
    return b

def est_page(clean_url):
    """
    Tente de déterminer si une url pointe vers une page web.

    clean_url est la cible d'un lien sans nom de domaine ni paramètre GET
    """
    # la détection est largement perfectible, par exemple :
    # http://www.exemple.com/un-super-article.com sera rejeté
    path, ext = os.path.splitext(clean_url)
    return (ext == "") or (ext in EXTENSION_WL)


def liste_url(texte):
    """
    Retourne la liste des url trouvées dans texte.
    """
    L = []
    i = 0
    while i != -1:
        url, i = prochaine_url(texte, i)
        # filtrer les urls de style/javascript/images....
        if est_action(url):
            continue # recherche de la prochaine url
        # recherche d'une éventuelle extension de fichier
        clean_url = urlparse(url).path # seulement le chemin
        if est_page(clean_url):
            # on retire l'éventuel fragment pour éviter de proposer plusieurs fois
            # la même page à des endroits différents. On ignore les hashtag
            L.append(urlparse(url)._replace(fragment="").geturl())
    return L


def trouve_liens(url):
    texte = html(url)
    L = liste_url(texte)
    L = normalise_urls(L, url)
    # tri par ordre alphabetique sans protocole
    L.sort(key=lambda e:e[e.index("//") + 2: ])
    return L


def nom_domaine(url):
    """
    retourne le nom de domaine complet d'une url.
    Par exemple pour "http://prepa.blaise-pascal.fr/16-17/ptsi/",
    retourne "prepa.blaise-pascal.fr".

    Les url doivent commencer par http:// ou https://
    """
    return urlparse(url).netloc

